using System;
using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using Random = UnityEngine.Random;

public class Gameplay : MonoBehaviour
{
	public static Gameplay instance;
	public int SavedCount{ get{ return _savedCount; } }
	public int KilledCount{ get{ return _killedCount; } }
	public List<GameObject> PlacedZombies;
	public static Action OnSaved;
	public float TimeBetweenSurvivorSpawn;
	public GameObject SurvivorPrefab;
	public int NumberOfSurvivorsToSpawn;
	public Transform SpawnPositionsContainer;
	public GameObject GameoverUI;
	public List<GameObject> NewZombies = new List<GameObject>();
	public AudioSource BackgroundMusic;
	public AudioSource SFX;
	public BattleTextManager BattleTextManager;

	int _killedCount;
	int _savedCount;
	Vector3[] ZombieStartPoints;
	IEnumerator _spawnRoutine;
	int amountDeadSinceSaved;

	void Awake()
	{
		Application.targetFrameRate = 60;
		instance = this;
		amountDeadSinceSaved = 0;
	}

	public void OnPlayPressed()
	{
		ResetGame();
		_spawnRoutine = SpawnSurvivors();
		StartCoroutine(_spawnRoutine);
	}

	void ResetGame()
	{
		BackgroundMusic.volume = 0.2f;
		_killedCount = 0;
		_savedCount = 0;
		amountDeadSinceSaved = 0;
		foreach (var zombie in NewZombies)
		{
			if(zombie != null) Destroy(zombie);
		}
		NewZombies.Clear();
	}

	public void OnSurvivorSaved()
	{
		BattleTextManager.ShowSurvivorSavedText();
		_savedCount++;
		DoSurvivorNumberChecks();
		if (OnSaved != null)
		{
			OnSaved();
		}
		amountDeadSinceSaved = 0;
	}

	public void OnSurvivorKilled()
	{
		_killedCount++;
		DoSurvivorNumberChecks();
		amountDeadSinceSaved++;

	}

	void DoSurvivorNumberChecks()
	{
		if (_killedCount + _savedCount == NumberOfSurvivorsToSpawn)
		{
			Gameover();
		}
	}

	void Update()
	{
		if (Input.GetKeyDown(KeyCode.A))
		{
			Gameover();
		}
		if(amountDeadSinceSaved > 3){
			Gameover ();
		}
	}

	void Gameover()
	{
		BackgroundMusic.volume = 0.05f;
		GameoverUI.SetActive(true);
		StopCoroutine(_spawnRoutine);

	}

	public void OnPlayAgain()
	{
		ResetGame();
		_spawnRoutine = SpawnSurvivors();
		StartCoroutine(_spawnRoutine);
	}

	IEnumerator SpawnSurvivors()
	{
		for (int i = 0; i < NumberOfSurvivorsToSpawn; i++)
		{
			Instantiate(SurvivorPrefab, GetRandomSpawnPos().position, Quaternion.identity);
			yield return new WaitForSeconds(TimeBetweenSurvivorSpawn);
		}
	}

	Transform GetRandomSpawnPos()
	{
		return SpawnPositionsContainer.GetChild(Random.Range(0, SpawnPositionsContainer.childCount));
	}
}
