﻿using System.Collections.Generic;
using UnityEngine;

public class LineDrawer : MonoBehaviour
{
	public LayerMask PlayerDrawLayer;
	List<Vector3> Points{ get{ return _survivor.Points; } }
	bool isDrawingPath;
	bool _isSurvivorSelected{ get{ return _survivor != null; } }
	Survivor _survivor;
	LineRenderer _lr;
	bool isSelected;

	void Awake()
	{
		_survivor = GetComponent<Survivor>();
		_lr = GetComponent<LineRenderer>();
	}

	void Update()
	{
		Vector3 mousePosInWorld = Camera.main.ScreenToWorldPoint(Input.mousePosition);
		
		if (Input.GetMouseButtonDown(0) || Input.touchCount > 0 && Input.touches[0].phase == TouchPhase.Began )
		{
			RaycastHit2D hitInfo = Physics2D.Raycast(mousePosInWorld,
				Vector2.zero, Mathf.Infinity, PlayerDrawLayer);
			if (hitInfo.collider != null && hitInfo.collider.GetComponentInChildren<Survivor>() == _survivor)
			{
				_survivor.ResetPoints();
				Points.Clear();
				Points.Add(_survivor.transform.position);
				isSelected = true;
			}
		}

		if (!isSelected) return;

		if (Input.GetMouseButton(0) || Input.touchCount > 0 && Input.touches[0].phase == TouchPhase.Moved)
		{
			Vector3 lastStepPosWorld = Points.Count > 0 ? Points[Points.Count - 1] : _survivor.transform.position;
			Vector3 mousePosWorld = Camera.main.ScreenToWorldPoint(Input.mousePosition);
			mousePosInWorld.z = -0.01f;
			
			float distanceFromLastPoint = Vector3.Distance(lastStepPosWorld, mousePosWorld);
			if (distanceFromLastPoint > 0.1f)
			{
				_survivor.AddPointToPath(mousePosInWorld);
				UpdateLineRenderer();
			}
		}

		if (Input.GetMouseButtonUp(0)|| Input.touchCount > 0 && Input.touches[0].phase == TouchPhase.Ended )
		{
			isSelected = false;
		}
	}

	public void UpdateLineRenderer()
	{
		_lr.SetVertexCount(_survivor.PointsRemaining.Count);
		_lr.SetPositions(_survivor.PointsRemaining.ToArray());
	}
}
