﻿using System.Collections.Generic;
using UnityEngine;

public class Survivor : MonoBehaviour
{
    Transform Camp;
    public GameObject ZombiePrefab;
	public GameObject BloodPrefab;
    public List<Vector3> Points = new List<Vector3>();
    public LineDrawer LineDrawer;
    public List<Vector3> PointsRemaining{
        get{ return Points.GetRange(currentPointsIndex, Points.Count - currentPointsIndex); }
    }
	public Sprite[] HumanSkins;
	public AudioClip[] Screams;
	public AudioClip[] SafeAndSounds;

	private AudioSource audio;


	public float lowPitchRange = .95f;              //The lowest a sound effect will be randomly pitched.
	public float highPitchRange = 1.05f;    


    bool hasReachedCamp;
    int currentPointsIndex = 0;
    bool hasPathToFollow;
    Vector3 targetPos;
    Vector3 currentDirection;
    Rigidbody2D _rb;
    public float WalkingSpeed = 0.025f;
    public float RunningSpeed = 0.05f;
    bool canDetectLevelExit = false;

    void Awake()
    {
        Camp = GameObject.FindGameObjectWithTag("Camp").transform;
        _rb = GetComponent<Rigidbody2D>();
    }

    void Start()
    {
        targetPos = Camp.position;
        currentDirection = Camp.position - transform.position;
        currentDirection.Normalize();
        float rot_z = Mathf.Atan2(currentDirection.y, currentDirection.x) * Mathf.Rad2Deg;
        transform.rotation = Quaternion.Euler(0f, 0f, rot_z);
		int rand = Random.Range (0, 6);
		GetComponent<SpriteRenderer> ().sprite = HumanSkins [rand];
		audio = Gameplay.instance.SFX;
    }
    
    void Update()
    {
        if (Points.Count > 0)
        {   
            targetPos = Points[currentPointsIndex];
            
            if (Vector3.Distance(transform.position, Points[currentPointsIndex]) < 0.2f)
            {
                // reached the next point.
                if (currentPointsIndex + 1 < Points.Count)
                {
                    currentPointsIndex++;
                    targetPos = Points[currentPointsIndex];
                    currentDirection = Points[currentPointsIndex] - transform.position;
                    currentDirection.Normalize();
                    float rot_z = Mathf.Atan2(currentDirection.y, currentDirection.x) * Mathf.Rad2Deg;
                    transform.rotation = Quaternion.Euler(0f, 0f, rot_z);
                    List<Vector3> PointsRemaining =
                        Points.GetRange(currentPointsIndex, Points.Count - currentPointsIndex);
                    LineDrawer.UpdateLineRenderer();
                }
                else
                {
                    targetPos = transform.position + currentDirection * 10f;
                    Points.Clear();
                }
            }
        }
        transform.parent.position = Vector2.MoveTowards(transform.position, targetPos, WalkingSpeed);
    }

    void OnTriggerStay2D(Collider2D other)
    {
        if (other.CompareTag("CampWalls"))
        {
            ResetPoints();
            targetPos = transform.position;
            LineDrawer.UpdateLineRenderer();
        }
    }

    void OnTriggerEnter2D(Collider2D other)
    {
        if (other.CompareTag("Zombie"))
        {
            ResetPoints();
            targetPos = transform.position;
            LineDrawer.UpdateLineRenderer();
            transform.parent.GetComponent<BoxCollider2D>().enabled = false;
			Instantiate (BloodPrefab, transform.parent.position, Quaternion.identity);
            Gameplay.instance.OnSurvivorKilled();
            Gameplay.instance.NewZombies.Add(Instantiate(ZombiePrefab, transform.parent.position, Quaternion.identity));

			int randSound = Random.Range (0, 3);
			float randomPitch = Random.Range(lowPitchRange, highPitchRange);
			audio.pitch = randomPitch;
			audio.clip = Screams[randSound];
			audio.PlayOneShot(audio.clip, 0.3f);
		
            Destroy(transform.parent.gameObject);
        }
        
        if (other.CompareTag("CampWalls"))
        {
            ResetPoints();
            targetPos = transform.position;
            LineDrawer.UpdateLineRenderer();
        }

        if (other.CompareTag("Safezone"))
        {
            Gameplay.instance.OnSurvivorSaved();
			int randSound = Random.Range (0, 7);
			float randomPitch = Random.Range(lowPitchRange, highPitchRange);
			audio.pitch = randomPitch;
			audio.clip = SafeAndSounds[randSound];
			audio.PlayOneShot(audio.clip, 0.5f);
            Destroy(transform.parent.gameObject);
        }
        
        if (other.CompareTag("LevelEnter") )
        {
            canDetectLevelExit = true;
        }
        
        if (other.CompareTag("LevelExit") && canDetectLevelExit)
        {
            Gameplay.instance.OnSurvivorKilled();
            Destroy(transform.parent.gameObject);
        }
    }

    public void AddPointToPath(Vector3 pointToAdd)
    {
        Points.Add(pointToAdd);
    }

    public void ResetPoints()
    {
        Points.Clear();
        currentPointsIndex = 0;
    }
}
