﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class Zombie : MonoBehaviour {

	public  LayerMask layer;
	public AudioClip[] zombieSee;
	public AudioClip[] zombieEat;

	private Collider2D following;
	private float timeSlowDown;
	private float timeIdle;
	private int randMove;
	private int randTime;

	private Collider2D lastFollowed;

	private bool movingAway;


	public float lowPitchRange = .55f;              //The lowest a sound effect will be randomly pitched.
	public float highPitchRange = .65f;            //The highest a sound effect will be randomly pitched.
	void Start()
	{		

		movingAway = false;
		
		timeIdle = 0;
		timeSlowDown = 0;
		following = null;
		lastFollowed = null;
		int randRotation = Random.Range (0, 360);

		transform.RotateAround (transform.position, new Vector3(0f, 0f, 1f),randRotation);
			
	}

	void Update()
	{

		if (!movingAway) {

			Collider2D[] hitColliders = Physics2D.OverlapCircleAll (transform.position, 3, layer);
			Collider2D nearest = null;
			float minDistance = 110000000;

			for (int i = 0; i < hitColliders.Length; ++i) {
				//hitColliders[i].SendMessage("AddDamage");

				float distance = Vector2.Distance (transform.position, hitColliders [i].transform.position);
				if (minDistance > distance) {
					minDistance = distance;
					nearest = hitColliders [i];
					following = nearest;
				}
			}
			if (following == null) {
				transform.GetComponent<SpriteRenderer> ().sprite = Resources.Load<Sprite> ("zombie_still");
				if (timeIdle == 0) {
					randMove = Random.Range (0, 4);
					randTime = Random.Range (0, 2);
					timeIdle += Time.deltaTime;
				} else if (timeIdle < 1) {
					if (randMove == 0) {
						transform.RotateAround (transform.position, new Vector3 (0f, 0f, 1f), 17 * Time.deltaTime);
						timeIdle += Time.deltaTime / randTime;
						transform.position += transform.right * Time.deltaTime * 0.1f;

					} else if (randMove == 1 || randMove == 3) {	
						timeIdle += Time.deltaTime / randTime;

						transform.position += transform.right * Time.deltaTime * 0.1f;

					} else if (randMove == 2) {
						transform.RotateAround (transform.position, new Vector3 (0f, 0f, -1f), 20 * Time.deltaTime);
						timeIdle += Time.deltaTime / randTime;
						transform.position += transform.right * Time.deltaTime * 0.1f;
			
				
					}
				} else {
					timeIdle = 0;
				}
			} else if (nearest != null) {
				transform.position = Vector2.MoveTowards (transform.position, nearest.transform.position, 0.007f);

				Vector3 diff = nearest.transform.position - transform.position;
				diff.Normalize ();

				float rot_z = Mathf.Atan2 (diff.y, diff.x) * Mathf.Rad2Deg;
				transform.rotation = Quaternion.Euler (0f, 0f, rot_z);
				transform.GetComponent<SpriteRenderer> ().sprite = Resources.Load<Sprite> ("zombie_walk");
				if(lastFollowed != following){
					lastFollowed = following;
					int randSound = Random.Range (0, 7);

					AudioSource audio = GetComponent<AudioSource> ();
					float randomPitch = Random.Range(lowPitchRange, highPitchRange);
					audio.pitch = randomPitch;
					audio.clip = zombieSee[randSound];
					audio.PlayOneShot(audio.clip, 0.8f);
				}

			} else {
				if (following.gameObject.activeSelf) {
					transform.GetComponent<SpriteRenderer> ().sprite = Resources.Load<Sprite> ("zombie_still");
					transform.position = Vector2.MoveTowards (transform.position, following.transform.position, 0.0025f);
					//transform.LookAt (nearest.transform.position, new Vector3(0,0,1));

					Vector3 diff = following.transform.position - transform.position;
					diff.Normalize ();

					float rot_z = Mathf.Atan2 (diff.y, diff.x) * Mathf.Rad2Deg;
					transform.rotation = Quaternion.Euler (0f, 0f, rot_z);
				} else {
					transform.GetComponent<SpriteRenderer> ().sprite = Resources.Load<Sprite> ("zombie_still");
					following = null;
					lastFollowed = null;
					//timeSlowDown =
				}
			
			}


		}

	}


	IEnumerator turnAroundAndMove(){
		movingAway = true;
		following = null;
		lastFollowed = null;
		float seconds = 0f;
		while (seconds < 1){
			transform.RotateAround (transform.position, new Vector3 (0f, 0f, 1f), 140 * Time.deltaTime);
			seconds += Time.deltaTime;
			yield return null;
		}
		while (seconds < 1.5){
			transform.position += transform.right * Time.deltaTime ;
			seconds += Time.deltaTime;
			yield return null;
		}
		movingAway = false;
	}

	void OnTriggerEnter2D (Collider2D c)
	{

		if(c.gameObject.name == "Survivor Sprite"){
			int randSound = Random.Range (0, 4);

			AudioSource audio = GetComponent<AudioSource> ();
			float randomPitch = Random.Range(lowPitchRange, highPitchRange);
			audio.pitch = randomPitch;
			audio.clip = zombieEat[randSound];
			audio.PlayOneShot(audio.clip, 0.6f);
		}
		if (!movingAway) {
			StartCoroutine (turnAroundAndMove ());

		}
	}




}
